@@ -20,24 +20,35 @@ struct mount_call
 	char path[0];
 };
 
-#define NFS_MAXFHSIZE 64
 
-struct nfs_fh
+/*
+ * The following structure is the NFS v3 on-the-wire file handle,
+ * as defined in rfc1813.
+ * This differs from the structure used by the kernel,
+ * defined in <linux/nfh3.h>: rfc has a long in network order,
+ * kernel has a short in native order.
+ * Both kernel and rfc use the name nfs_fh; kernel name is
+ * visible to user apps in some versions of libc.
+ * Use different name to avoid clashes.
+ */
+#define NFS_MAXFHSIZE_WIRE 64
+struct nfs_fh_wire
 {
 	__u32 size;
-	char data[NFS_MAXFHSIZE];
+	char data[NFS_MAXFHSIZE_WIRE];
 } __attribute__((packed));
 
+
 struct mount_reply
 {
 	struct rpc_reply reply;
 	__u32 status;
-	struct nfs_fh fh;
+	struct nfs_fh_wire fh;
 } __attribute__((packed));
 
 #define MNT_REPLY_MINSIZE (sizeof(struct rpc_reply) + sizeof(__u32))
 
-static void get_ports(__u32 server, const struct nfs_mount_data *data)
+static int get_ports(__u32 server, const struct nfs_mount_data *data)
 {
 	__u32 nfs_ver, mount_ver;
 	__u32 proto;
@@ -61,7 +72,7 @@ static void get_ports(__u32 server, const struct nfs_mount_data *data)
 				fprintf(stderr, "NFS over TCP not "
 					"available from %s\n",
 					inet_ntoa(addr));
-				exit(1);
+				return -1;
 			}
 			nfs_port = NFS_PORT;
 		}
@@ -73,6 +84,7 @@ static void get_ports(__u32 server, const struct nfs_mount_data *data)
 		if (mount_port == 0)
 			mount_port = MOUNT_PORT;
 	}
+	return 0;
 }
 
 static inline int pad_len(int len)
@@ -151,9 +163,9 @@ static int mount_call(__u32 proc, __u32 version,
 
 	memset(mnt_call, 0, sizeof(*mnt_call));
 
-	mnt_call->rpc.program = __constant_htonl(NFS_MNT_PROGRAM);
-	mnt_call->rpc.prog_vers = __constant_htonl(version);
-	mnt_call->rpc.proc = __constant_htonl(proc);
+	mnt_call->rpc.program = htonl(NFS_MNT_PROGRAM);
+	mnt_call->rpc.prog_vers = htonl(version);
+	mnt_call->rpc.proc = htonl(proc);
 	mnt_call->path_len = htonl(path_len);
 	memcpy(mnt_call->path, path, path_len);
 
@@ -165,8 +177,12 @@ static int mount_call(__u32 proc, __u32 version,
 	if (rpc_call(clnt, &rpc) < 0)
 		goto bail;
 
+	if (proc != MNTPROC_MNT) {
+		goto done;
+	}
+
 	if (rpc.reply_len < MNT_REPLY_MINSIZE) {
-		fprintf(stderr, "incomplete reply: %d < %d\n",
+		fprintf(stderr, "incomplete reply: %zu < %zu\n",
 			rpc.reply_len, MNT_REPLY_MINSIZE);
 		goto bail;
 	}
@@ -249,7 +265,9 @@ int nfs_mount(const char *pathname, const char *hostname,
 	int ret = 0;
 	int mountflags;
 
-	get_ports(server, data);
+	if (get_ports(server, data) != 0) {
+		goto bail;
+	}
 
 	dump_params(server, path, data);
 
